/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/* eslint-disable import/no-unresolved, import/no-extraneous-dependencies */


const path = require('path');
const webpack = require('webpack');
const userConfig = require('../../../config.ts');
const packageJson = require('../../../package.json');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const ClearTerminalPlugin = require('./ClearTerminalPlugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');


// Contains all webpack's context-agnostic production configuration
// (which is common to both front-end and back-end projects types).
const baseConfig = {
  target: userConfig.target,
  bail: true,
  cache: false,
  mode: 'production',
  context: userConfig.srcPath,
  devtool: 'source-map',
  entry: userConfig.entry,
  resolve: {
    extensions: ['.json', '.ts', '.tsx', '.js', '*'],
  },
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        include: [userConfig.srcPath],
        use: [
          // `awesome-typescript-loader` is then used, to compile Typescript into Javascript.
          {
            loader: 'awesome-typescript-loader',
            options: {
              useCache: false,
            },
          },
          // `tslint-loader` is first used, to lint the code.
          {
            loader: 'tslint-loader',
            options: {
              emitErrors: true,
              failOnHint: true,
              typeCheck: true,
            },
          },
        ],
      },
      // Stylesheets.
      {
        test: /\.css$/,
        use: [
          MiniCssExtractPlugin.loader,
          // `css-loader` resolves paths in CSS and adds assets as dependencies.
          {
            loader: 'css-loader',
            options: { sourceMap: true, minimize: true },
          },
          // `postcss-loader` is used to autoprefix CSS to ensure compatibility with all browsers.
          {
            loader: 'postcss-loader',
            options: {
              config: { path: path.resolve(__dirname, 'postcss.config.js') },
              sourceMap: true,
            },
          },
        ],
      },
      // Audio and video files.
      {
        test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10 * 1024,
          name: 'media/[name].[hash:7].[ext]',
        },
      },
      // Images files.
      {
        test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10 * 1024,
          name: 'images/[name].[hash:7].[ext]',
        },
      },
      // Fonts files.
      {
        test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
        loader: 'url-loader',
        options: {
          limit: 10 * 1024,
          name: 'fonts/[name].[hash:7].[ext]',
        },
      },
    ],
  },
  plugins: [
    // Clears terminal between each compilation.
    new ClearTerminalPlugin(),
    // Includes copyright comment on top of each generated file.
    new webpack.BannerPlugin({
      banner: userConfig.banner,
      raw: true,
    }),
  ],
};


// Additional configuration for front-end projects.
if (userConfig.target === 'web') {
  baseConfig.output = {
    path: `${userConfig.distPath}/assets`,
    filename: 'scripts/[name].[chunkhash].js',
  };
  baseConfig.plugins = baseConfig.plugins.concat([
    // Extracts all the CSS into a specific file.
    new MiniCssExtractPlugin({
      filename: 'styles/[name].[chunkhash].css',
    }),
  ]);
  baseConfig.node = {
    // Prevents webpack from injecting useless setImmediate polyfill because Vue
    // source contains it (although only uses it if it's native).
    setImmediate: false,
    // Prevents webpack from injecting mocks to Node native modules
    // that does not make sense for the client.
    dgram: 'empty',
    fs: 'empty',
    net: 'empty',
    tls: 'empty',
    child_process: 'empty',
  };
  baseConfig.optimization = {
    splitChunks: {
      chunks: 'all',
    },
    runtimeChunk: true,
    minimizer: [
      new UglifyJsPlugin({
        sourceMap: true,
        parallel: true,
        uglifyOptions: {
          ecma: 6,
        },
      }),
    ],
  };


// Additional configuration for back-end projects.
} else {
  baseConfig.externals = Object.keys(packageJson.dependencies).reduce(
    (externals, dependency) => Object.assign(externals, { [dependency]: dependency }),
    {},
  );
  baseConfig.output = {
    path: userConfig.distPath,
    filename: '[name].js',
    library: packageJson.name,
    libraryTarget: 'commonjs2',
  };
  // Disables all the nodeJS polyfills and mocks only if target is `node`.
  baseConfig.node = {
    console: false,
    global: false,
    process: false,
    __filename: false,
    __dirname: false,
    Buffer: false,
    setImmediate: false,
  };
  baseConfig.optimization = {
    minimizer: [
      new UglifyJsPlugin({
        sourceMap: true,
        parallel: true,
        uglifyOptions: {
          ecma: 6,
          mangle: false,
        },
      }),
    ],
  };
}


module.exports = baseConfig;
